﻿using NMemory;
using NMemory.Indexes;
using NMemory.Tables;

namespace TestProject;

/// <summary>
/// 테스트 데이터베이스
/// </summary>
public class TestDatabase : Database
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Property
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 그룹 테이블 - GroupTable

    /// <summary>
    /// 그룹 테이블
    /// </summary>
    public ITable<Group> GroupTable { get; private set; }

    #endregion
    #region 회원 테이블 - MemberTable

    /// <summary>
    /// 회원 테이블
    /// </summary>
    public ITable<Member> MemberTable { get; private set; }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 생성자 - TestDatabase()

    /// <summary>
    /// 생성자
    /// </summary>
    public TestDatabase()
    {
        Table<Group , int> groupTable  = Tables.Create<Group , int>(g => g.ID, new IdentitySpecification<Group >(g => g.ID, 1, 1));
        Table<Member, int> memberTable = Tables.Create<Member, int>(m => m.ID, new IdentitySpecification<Member>(m => m.ID, 1, 1));

        GroupTable  = groupTable;
        MemberTable = memberTable;

        RelationOptions options = new RelationOptions(cascadedDeletion : true);

        IIndex<Member, int?> memberGroupIDIndex = memberTable.CreateIndex(new RedBlackTreeIndexFactory(), m => m.GroupID);

        Tables.CreateRelation
        (
            groupTable.PrimaryKeyIndex,
            memberGroupIDIndex,
            x =>  x ?? -1,
            x => x,
            options
        );
    }

    #endregion
}